﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;

namespace CKMail
{
    public partial class F12_SetSound : Form
    {
        /// <summary> コンストラクター
        /// </summary>
        public F12_SetSound()
        {
            InitializeComponent();
        }

        const int NUM_FILE = 4;

        private bool _isChanged = false;
        private CMidi _cmidi;
        private int _rbtnIndex = 0;
        private RadioButton[] _rbtn = new RadioButton[NUM_FILE];

        /// <summary> [カスタマイズ]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCustom_Click(object sender, EventArgs e)
        {
            if (btnStop.Enabled) StopMidi();

            try
            {
                string file1 = GetFilename(_rbtnIndex);
                string folder1 = (File.Exists(file1)) ? Path.GetDirectoryName(file1): "C:\\";

                openFileDlg.Filter = "midファイル(*.mid)|*.mid";
                openFileDlg.InitialDirectory = folder1;
                openFileDlg.FileName = String.Empty;
                if (openFileDlg.ShowDialog() == DialogResult.OK)
                {                                
                    string msg = "[" + _rbtn[_rbtnIndex].Text + "]を別のmidファイルに変更します。";
                    if (CC.MsgCanceled(msg)) return;

                    string filename = openFileDlg.FileName;
                    switch (_rbtnIndex)
                    {
                        case 0:
                            Properties.Settings.Default.MidFile0 = filename;
                            break;
                        case 1:
                            Properties.Settings.Default.MidFile1 = filename;
                            break;
                        case 2:
                            Properties.Settings.Default.MidFile2 = filename;
                            break;
                        case 3:
                            Properties.Settings.Default.MidFile3 = filename;
                            break;
                    }
                    CC.MidFile = filename;
                    Properties.Settings.Default.NoSound = chkNoSound.Checked;
                    Properties.Settings.Default.MidFileIdx = _rbtnIndex;
                    Properties.Settings.Default.Save();
                    _isChanged = false;
                    CC.ShowMsg("効果音のカスタマイズを設定しました。", CC.MSG_INF);
                }           
            }
            catch (Exception ex)
            {
                    CC.ShowMsg(ex.ToString(), CC.MSG_EXC);
            }
        }

        /// <summary> [OK]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (_isChanged)
            {
                // 設定実行の確認
                if (CC.MsgCanceled("この設定を保存します。")) return;                  

                // 設定の保存
                Properties.Settings.Default.NoSound = chkNoSound.Checked;
                Properties.Settings.Default.MidFileIdx = _rbtnIndex;
                Properties.Settings.Default.Save();
            }
            this.Close();
        }

        /// <summary> [停止]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStop_Click(object sender, EventArgs e)
        {
            StopMidi();
        }

        /// <summary> [試聴]ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTest_Click(object sender, EventArgs e)
        {
            string filename = GetFilename(_rbtnIndex);
            if (filename.Length == 0)
            {
                CC.ShowMsg("midファイルが選択されていません。", CC.MSG_EXC);
                return;
            }            
            if (!File.Exists(filename))
            {
                CC.ShowMsg("midファイルが設定されていません。", CC.MSG_EXC);
                return;
            }
            
            // midファイルを開く
            btnTest.Enabled = false;
            btnCustom.Enabled = false;
            Application.DoEvents();
            _cmidi = new CMidi(filename);
            if (_cmidi.CanStartMidi())
            {
                btnStop.Enabled = true;
            }
            else
            {
                btnTest.Enabled = true;
                btnCustom.Enabled = true;
                CC.ShowMsg("midファイルを開くことができません。", CC.MSG_EXC);
            }            
        }

        /// <summary> [chkNoSound_CheckedChanged]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkNoSound_CheckedChanged(object sender, EventArgs e)
        {
            if (btnStop.Enabled) StopMidi();

            panel2.Enabled = !(chkNoSound.Checked);
            _isChanged = true;
        }

        /// <summary> [Form_Closing]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Closing(object sender, FormClosingEventArgs e)
        {
            if (btnStop.Enabled) StopMidi();
            if (_cmidi != null) _cmidi = null;
        }

        /// <summary> [Form_Load]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form_Load(object sender, EventArgs e)
        {
            CC.SetForm(this);
            InitializeForm();
            _isChanged = false;
        }

        /// <summary> [Rdo_CheckedChanged]イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Rdo_CheckedChaged(object sender, EventArgs e)
        {
            RadioButton rbtn = (RadioButton)sender;

            if (rbtn.Checked)
            {
                string rname = rbtn.Name;
                _rbtnIndex = Int32.Parse(rname.Substring(rname.Length - 1, 1));
                CC.MidFile = GetFilename(_rbtnIndex);            
                if (btnStop.Enabled) StopMidi();
                _isChanged = true;
            }
        }


        /// <summary> [GetFilename]メソッド
        ///  [btnCustom_Click],[btnTest_Click]から呼出し
        /// </summary>
        /// <param name="vindex">_rbtnのindex</param>
        /// <returns>Setting情報中のfilename</returns>
        private string GetFilename(int vindex)
        {
            string filename = String.Empty;

            switch (vindex)
            {
                case 0:
                    filename = Properties.Settings.Default.MidFile0;
                    break;
                case 1:
                    filename = Properties.Settings.Default.MidFile1;
                    break;
                case 2:
                    filename = Properties.Settings.Default.MidFile2;
                    break;
                default:
                    filename = Properties.Settings.Default.MidFile3;
                    break;
            }
            return filename;
        }

        /// <summary> [InitializeForm]メソッド
        /// [Form_Load]から呼出し
        /// </summary>
        private void InitializeForm()
        {
            btnStop.Enabled = false;
            
            foreach (Control ctrl in panel2.Controls)
            {
                if (ctrl.Name.Substring(0, 3).ToLower() == "rdo")
                {
                    string rname = ctrl.Name;
                    int num = Int32.Parse(rname.Substring(rname.Length - 1, 1));
                    _rbtn[num] = (RadioButton)ctrl;
                }

            }

            _rbtn[Properties.Settings.Default.MidFileIdx].Checked = true;
            if (Properties.Settings.Default.NoSound)
            {
                chkNoSound.Checked = true;
            }
            else
            {
                //
            }
        }

        /// <summary> [StopMdi]メソッド
        /// [btnStop_Click]、[Form_Closing]、[RdoCheckedChanged]から呼出し
        /// </summary>
        private void StopMidi()
        {
            if (_cmidi != null)
            {
                _cmidi.StopMidi();
                btnTest.Enabled = true;
                btnCustom.Enabled = true;
                btnStop.Enabled = false;
            }
        }

    }
}
